@@ -1,5 +1,7 @@
 from typing import Dict, overload, List, Union, Tuple, Generator, Iterable
+from numpy import character, integer
 
+from amulet_nbt import TAG_Byte, TAG_Int, TAG_Short, TAG_Long, TAG_String
 from amulet.api.data_types import Int, BiomeType
 from amulet.api.registry.base_registry import BaseRegistry
 
@@ -16,7 +18,7 @@ class BiomeManager(BaseRegistry):
         self._index_to_biome: List[BiomeType] = []
         self._biome_to_index: Dict[BiomeType, int] = {}
         for biome in biomes:
-            # if a list is given it is assumed that the block palette will be the same size as the list.
+            # if a list is given it is assumed that the biome palette will be the same size as the list.
             # Ensure that if a value is duplicated it will appear twice in the list
             assert isinstance(biome, str), f"biome must be a string. Got {biome}"
             if biome not in self._biome_to_index:
@@ -85,27 +87,43 @@ class BiomeManager(BaseRegistry):
     def __getitem__(self, item: Int) -> BiomeType:
         ...
 
+    @overload
+    def __getitem__(
+        self, item: Iterable[Union[Int, BiomeType]]
+    ) -> List[Union[BiomeType, Int]]:
+        ...
+
     def __getitem__(self, item):
         """
         If a string is passed to this function, it will return the internal ID/index of the biome.
 
         If an int is given, this method will return the biome string at that specified index.
 
+        >>> level.biome_palette[biome]
+        7
+        >>> level.biome_palette[7]
+        biome
+
         :param item: The string or int to get the mapping data of
         :return: An int if a string was supplied, a string if an int was supplied
         :raises KeyError if the requested item is not present.
         """
         try:
-            if isinstance(item, str):
-                return self._biome_to_index[item]
-            return self._index_to_biome[item]
-
+            return self._get_item(item)
         except (KeyError, IndexError):
             raise KeyError(
                 f"There is no {item} in the BiomeManager. "
                 f"You might want to use the `get_add_biome` function for the biome before accessing them."
             )
 
+    def _get_item(self, item):
+        if isinstance(item, (str, character, TAG_String)):
+            return self._biome_to_index[str(item)]
+        elif isinstance(item, (int, integer, TAG_Byte, TAG_Short, TAG_Int, TAG_Long)):
+            return self._index_to_biome[int(item)]
+        # if it isn't an int or string assume an iterable of the above.
+        return [self._get_item(i) for i in item]
+
     def get_add_biome(self, biome: BiomeType) -> int:
         """
         Adds a biome string to the internal biome string/ID mappings.
