@@ -1,6 +1,8 @@
 from __future__ import annotations
+from typing import Dict, Iterable, List, Tuple, overload, Generator, Union
+from numpy import integer
 
-from typing import Dict, Iterable, List, Tuple, overload, Generator
+from amulet_nbt import TAG_Byte, TAG_Int, TAG_Short, TAG_Long
 
 from amulet.api.data_types import Int
 from amulet.api.block import Block
@@ -87,27 +89,41 @@ class BlockManager(BaseRegistry):
     def __getitem__(self, item: Int) -> Block:
         ...
 
+    @overload
+    def __getitem__(self, item: Iterable[Union[Int, Block]]) -> List[Union[Block, Int]]:
+        ...
+
     def __getitem__(self, item):
         """
         If a Block object is passed to this function, it will return the internal ID/index of the blockstate.
 
         If an int is given, this method will return the Block object at that specified index.
 
+        >>> level.block_palette[block]
+        7
+        >>> level.block_palette[7]
+        block
+
         :param item: The Block object or int to get the mapping data of
         :return: An int if a Block object was supplied, a Block object if an int was supplied
         :raises KeyError if the requested item is not present.
         """
         try:
-            if isinstance(item, Block):
-                return self._block_to_index_map[item]
-
-            return self._index_to_block[item]
+            return self._get_item(item)
         except (KeyError, IndexError):
             raise KeyError(
                 f"There is no {item} in the BlockManager. "
                 f"You might want to use the `get_add_block` function for your blocks before accessing them."
             )
 
+    def _get_item(self, item):
+        if isinstance(item, Block):
+            return self._block_to_index_map[item]
+        elif isinstance(item, (int, integer, TAG_Byte, TAG_Short, TAG_Int, TAG_Long)):
+            return self._index_to_block[int(item)]
+        # if it isn't an Block or int assume an iterable of the above.
+        return [self._get_item(i) for i in item]
+
     def get_add_block(self, block: Block) -> int:
         """
         Adds a Block object to the internal Block object/ID mappings.
