@@ -132,7 +132,7 @@ class AnvilFormat(WorldFormatWrapper):
 
     @property
     def level_name(self) -> str:
-        return str(self.root_tag["Data"].get("LevelName", ""))
+        return str(self.root_tag.get("Data", {}).get("LevelName", ""))
 
     @level_name.setter
     def level_name(self, value: str):
@@ -140,7 +140,7 @@ class AnvilFormat(WorldFormatWrapper):
 
     @property
     def last_played(self) -> int:
-        return self.root_tag["Data"]["LastPlayed"].value
+        return int(self.root_tag.get("Data", {}).get("LastPlayed", 0))
 
     @property
     def game_version_string(self) -> str:
@@ -204,7 +204,7 @@ class AnvilFormat(WorldFormatWrapper):
                 path, mcc=self._mcc_support
             )
             self._dimension_name_map[dimension_name] = relative_dimension_path
-            bounds = None
+            bounds = DefaultSelection
             if self.version >= 2709:  # This number might be smaller
 
                 def get_recursive(obj: nbt.TAG_Compound, *keys):
@@ -230,6 +230,17 @@ class AnvilFormat(WorldFormatWrapper):
                     dimension_type: str = dimension_type.value
                     if ":" in dimension_type:
                         namespace, base_name = dimension_type.split(":", 1)
+                        if (
+                            self.version >= 2834
+                            and namespace == "minecraft"
+                            and base_name == "overworld"
+                        ):
+                            bounds = SelectionGroup(
+                                SelectionBox(
+                                    (-30_000_000, -64, -30_000_000),
+                                    (30_000_000, 320, 30_000_000),
+                                )
+                            )
                         dimension_path = (
                             f"data/{namespace}/dimension_type/{base_name}.json"
                         )
@@ -297,8 +308,6 @@ class AnvilFormat(WorldFormatWrapper):
                         )
                     )
 
-            if bounds is None:
-                bounds = DefaultSelection
             self._bounds[dimension_name] = bounds
 
     def _get_interface_key(
@@ -607,11 +616,40 @@ class AnvilFormat(WorldFormatWrapper):
             dimension_str = OVERWORLD
         if dimension_str not in self._dimension_name_map:
             dimension_str = OVERWORLD
+
+        # get the players position
+        pos_data = player_nbt.get("Pos")
+        if (
+            isinstance(pos_data, nbt.TAG_List)
+            and len(pos_data) == 3
+            and pos_data.list_data_type == nbt.TAG_Double.tag_id
+        ):
+            position = tuple(map(float, pos_data))
+            position = tuple(
+                p if -100_000_000 <= p <= 100_000_000 else 0.0 for p in position
+            )
+        else:
+            position = (0.0, 0.0, 0.0)
+
+        # get the players rotation
+        rot_data = player_nbt.get("Rotation")
+        if (
+            isinstance(rot_data, nbt.TAG_List)
+            and len(rot_data) == 2
+            and rot_data.list_data_type == nbt.TAG_Double.tag_id
+        ):
+            rotation = tuple(map(float, rot_data))
+            rotation = tuple(
+                p if -100_000_000 <= p <= 100_000_000 else 0.0 for p in rotation
+            )
+        else:
+            rotation = (0.0, 0.0)
+
         return Player(
             player_id,
             dimension_str,
-            tuple(map(lambda t: t.value, player_nbt["Pos"])),
-            tuple(map(lambda t: t.value, player_nbt["Rotation"])),
+            position,
+            rotation,
         )
 
     def _get_raw_player_data(self, player_id: str) -> nbt.NBTFile:
