@@ -136,7 +136,7 @@ class LevelDBFormat(WorldFormatWrapper):
             self._version = self._get_version()
         return self._version
 
-    def _get_version(self) -> Tuple[int, ...]:
+    def _get_version(self) -> VersionNumberTuple:
         """
         The version the world was last opened in.
 
@@ -213,21 +213,12 @@ class LevelDBFormat(WorldFormatWrapper):
             else:
                 chunk_version = raw_chunk_data.get(b"v", b"\x00")[0]
         else:
-            if (
-                self.root_tag.get("experiments", {})
-                .get("caves_and_cliffs_internal", nbt.TAG_Byte())
-                .value
-            ):
-                chunk_version = 26
-            elif (
+            chunk_version = game_to_chunk_version(
+                self.max_world_version[1],
                 self.root_tag.get("experiments", {})
                 .get("caves_and_cliffs", nbt.TAG_Byte())
-                .value
-            ):
-                # TODO: handle this properly when the chunk version is better understood for the 1.18 chunk
-                chunk_version = 25
-            else:
-                chunk_version = game_to_chunk_version(self.max_world_version[1])
+                .value,
+            )
         return self.platform, chunk_version  # TODO: work out a valid default
 
     def _decode(
@@ -269,6 +260,7 @@ class LevelDBFormat(WorldFormatWrapper):
             if (
                 experiments.get("caves_and_cliffs", nbt.TAG_Byte()).value
                 or experiments.get("caves_and_cliffs_internal", nbt.TAG_Byte()).value
+                or self.version >= (1, 18)
             ):
                 self._bounds[OVERWORLD] = SelectionGroup(
                     SelectionBox(
@@ -291,16 +283,20 @@ class LevelDBFormat(WorldFormatWrapper):
                     (-30_000_000, 0, -30_000_000), (30_000_000, 256, 30_000_000)
                 )
             )
+        except OSError as e:
+            self._is_open = self._has_lock = False
+            raise LevelDBException(
+                "It looks like this world is from the marketplace.\nThese worlds are encrypted and cannot be edited."
+            ) from e
         except LevelDBException as e:
             msg = str(e)
+            self._is_open = self._has_lock = False
             # I don't know if there is a better way of handling this.
-            self._is_open = False
-            self._has_lock = False
             if msg.startswith("IO error:") and msg.endswith(": Permission denied"):
                 traceback.print_exc()
                 raise LevelDBException(
                     f"Failed to load the database. The world may be open somewhere else.\n{msg}"
-                )
+                ) from e
             else:
                 raise e
 
@@ -428,11 +424,40 @@ class LevelDBFormat(WorldFormatWrapper):
             }[dimension.value]
         else:
             dimension_str = OVERWORLD
+
+        # get the players position
+        pos_data = player_nbt.get("Pos")
+        if (
+            isinstance(pos_data, nbt.TAG_List)
+            and len(pos_data) == 3
+            and pos_data.list_data_type == nbt.TAG_Float.tag_id
+        ):
+            position = tuple(map(float, pos_data))
+            position = tuple(
+                p if -100_000_000 <= p <= 100_000_000 else 0.0 for p in position
+            )
+        else:
+            position = (0.0, 0.0, 0.0)
+
+        # get the players rotation
+        rot_data = player_nbt.get("Rotation")
+        if (
+            isinstance(rot_data, nbt.TAG_List)
+            and len(rot_data) == 2
+            and rot_data.list_data_type == nbt.TAG_Float.tag_id
+        ):
+            rotation = tuple(map(float, rot_data))
+            rotation = tuple(
+                p if -100_000_000 <= p <= 100_000_000 else 0.0 for p in rotation
+            )
+        else:
+            rotation = (0.0, 0.0)
+
         return Player(
             player_id,
             dimension_str,
-            tuple(map(lambda t: t.value, player_nbt["Pos"])),
-            tuple(map(lambda t: t.value, player_nbt["Rotation"])),
+            position,
+            rotation,
         )
 
     def _get_raw_player_data(self, player_id: str) -> nbt.NBTFile:
