@@ -6,9 +6,12 @@ from .anvil_2203 import (
 
 
 class Anvil2529Interface(Anvil2203Interface):
-    def __init__(self):
-        super().__init__()
-        self._set_feature("long_array_format", "1.16")
+    """
+    Packed long arrays switched to a less dense format
+    Before the long array was just a bit stream but it is now separate longs. The upper bits are unused in some cases.
+    """
+
+    LongArrayDense = False
 
     @staticmethod
     def minor_is_valid(key: int):
