@@ -1,11 +1,9 @@
-from __future__ import annotations
-
 from .leveldb_25 import (
-    LevelDB25Interface,
+    LevelDB25Interface as ParentInterface,
 )
 
 
-class LevelDB26Interface(LevelDB25Interface):
+class LevelDB26Interface(ParentInterface):
     def __init__(self):
         super().__init__()
 
